function plugin_tag_function(obj, data)
% default tagging function

    %% FIND INDICES
    if obj.TAG_RETURN_FAKE_INDICES == 1
        indices = 1024;
        indices_off = 1024;
    else
        [indices, indices_off] = obj.find_indices(data);
        if obj.TAG_QUIT == 1
            disp('The error above occoured while calling find_indices().');
            return;
        end
    end

    %% CLEAN UP INDICES
    [indices, indices_off] = clean_up_indices(obj, indices, indices_off);
    if obj.TAG_QUIT == 1
        return;
    end
    
    %% TRANSFORM INDICES into SAMPLES
    obj.transform_indices(indices, indices_off);
    if obj.TAG_QUIT == 1
        return;
    end
    
    %% check for minimum window sizes
    % be careful here:
    % if TAG_RETURN_WHOLE_BUFFERS_ONLY = 1, then some of these indices
    % might get kicked out in fb_abstract.tag.extract_data()
    if obj.TAG_MIN_WINDOW_SIZE ~= 0 && numel(obj.TAG_INDICES) > 2
        
        % 1) kill small noise windows first (by merging neighboring signal
        % windows
        % 2) check if resulting signal windows are too small -> remove
        % them.
        
        % 1) remove noise periods
        noise_periods = obj.TAG_INDICES(2:end) - obj.TAG_INDICES_OFF(1:end-1);
        too_short_noise_periods = 1 + find(noise_periods < obj.TAG_MIN_WINDOW_SIZE);
        
        while numel(too_short_noise_periods) > 0
            last_element = too_short_noise_periods(end);
            obj.TAG_INDICES_OFF(last_element - 1) = obj.TAG_INDICES_OFF(last_element);
            obj.TAG_INDICES_OFF(last_element) = [];
            obj.TAG_INDICES(last_element) = [];
            too_short_noise_periods(end) = [];
        end
        
        % 2) kill too short signal periods
        signal_periods = obj.TAG_INDICES_OFF - obj.TAG_INDICES;
        too_short_signal_periods = find(signal_periods < obj.TAG_MIN_WINDOW_SIZE);
        
        if ~isempty(too_short_signal_periods)
            obj.TAG_INDICES(too_short_signal_periods) = [];
            obj.TAG_INDICES_OFF(too_short_signal_periods) = [];
        end
        
    end
    

    %% REMOVE OVERLAPPING INDICES
    obj.remove_overlapping_indices();
    
    %% CHECK IF AT LEAST ONE THRESHOLD CROSSING WAS DETECTED
    nbr_TAG_INDICES = size(obj.TAG_INDICES, 2);
    if nbr_TAG_INDICES > 0
        obj.TAG_FOUND = 1;
    end

    %% QUIT IF USER DOES NOT WANT TO GET THE DATA AROUND INDICES
    if obj.TAG_EXTRACT_BUFFERS == 0
        return;
    end

    %% EXTRACT DATA AROUND TAG_INDICES
    obj.extract_data(data, nbr_TAG_INDICES);

end
%% EOF
