function run_checks(obj, data)
% RUN checks before tagging

    %% assume that some of the checks will fail.
    obj.TAG_QUIT = 1;
    obj.initialize_result_fields();
    
    %% check whether user has set a threshold
    if isnan(obj.TAG_THRESHOLD)
        disp('"tag_data() quit, because no ''TAG_THRESHOLD'' is set.');
        return;
    end

    %% if TAG_MULTIDIM_DATA_MODE_ENABLED is 0, make sure data is a 1 times X vector
    size_data = size(data);
    nbr_channels = size_data(1);
    if obj.TAG_MULTIDIM_DATA_MODE_ENABLED == 0 && (nbr_channels ~= 1 || size_data(2) < nbr_channels)
        disp('"data" needs to be a row vector of size (1, n), where n is the number of samples.');
        return;
    end

    %% check if TAG_MULTIDIM_USE_CHANNEL is not exceeding the total number of channels
    if obj.TAG_MULTIDIM_DATA_MODE_ENABLED == 1 && obj.TAG_MULTIDIM_USE_CHANNEL > nbr_channels
        disp(['The channel you''ve selected to run the tagging on (' num2str(obj.TAG_MULTIDIM_USE_CHANNEL) '), is exceeding the total number of channels (' num2str(nbr_channels) ').']);
        return;
    end

    %% CHECK FOR CORRECT 'data" formats
    if ~isinteger(data) && ~isfloat(data)
        disp('"tag_data() quit, because "data" was neither an integer nor a double array.');
        return;
    end

    if ~isinteger(data) && obj.TAG_DATA_TYPE == 0
        disp('"tag_data() quit, because "data" was not an integer array.');
        return;
    end

    if ~isfloat(data) && obj.TAG_DATA_TYPE == 1
        disp('"tag_data() quit, because "data" was not a double array.');
        return;
    end

    %% we passed all checks - no need to quit.
    obj.TAG_QUIT = 0;

end
%% EOF
