function GetWav(varargin) 
%Parameters: [Filename to decode (*.dat)],
%      [number of generated WAV files (optional, default = 1)],
%      [Destination directory (optional, default = *.dat file directory)],
%      [Microphone amplification coefficient (optional, default = 80],
%      [Accelerometer amplification coefficient (optional, default = 80],
%      [Record start in default Matlab format
%      (optional, default = datenum([2013 12 13 07 00 00]) ]

CurrentDirectory = pwd;
%OrigDataDir = 'Data_original\2013-12-13';
Param.nargin = nargin;
Param.varargin = varargin;
if nargin==0
  OrigDataDir = 'Data_original\2013-12-13';
  Name = 'g2k8';
else
  InputLine = varargin{1};  
  [pathstr, Name, ext] = fileparts(InputLine);
  OrigDataDir = pathstr;
end  
Param.scanrateDesired = [19200]; %was [19200 32000]  
if (nargin>1)&&(~isempty(varargin{2}))
  Param.NFiles = varargin{2};  
else
  Param.NFiles = 1; %was 12; number of files in which the record can be split for convenience of further processing (in "full record" mode only)  
end
if isempty(strfind(OrigDataDir,':')) %no full path is given
  Param.StartDir = [CurrentDirectory filesep OrigDataDir];
else %full path is given
  Param.StartDir = OrigDataDir;  
end    
Param.WavFile = true; %default: false
Param.LVDFile = false; %default: true
Param.WAVCoeff = [80 80]; %This is to increase loudness 80x for better hearing, [microphone accelerometer]
if (nargin>3)&&(~isempty(varargin{4}))
  Param.WAVCoeff(1) = varargin{4};  
end    
if (nargin>4)&&(~isempty(varargin{5}))
  Param.WAVCoeff(2) = varargin{5};  
end
if (nargin>5)&&(~isempty(varargin{6}))
  Param.SynchronizationTime = varargin{6};
else
  Param.SynchronizationTime = datenum([2013 12 13 07 00 00]);    %is used when Synchronization = false; real start is unknown    
end
Param.Synchronization = false; %default: true
  Param.AnimalName = Name;%'g2k8'; %is used when Synchronization = false; animal name can't be taken from NI LVD

Param.LastOutputLVDNumber = 0;  %0- to start LVD numbering from 1 
Param.NStartPulses = 2; %2- manual synchronization, 1 - automatic start
Param.SongSelection = [4]; %[1 2 3 4] %1 - done by NI microphone, 2 - HeadMic, 3 - Accelerometer, 4 - full record
%Param.scanrateDesired = [19200]; %was [19200 32000]
Param.MyGPU = 2; %0 - don't use GPU (very slow), if selected GPU is not present, GPU with the highest number will be used (or no GPU)
Param.Playback = false;%true
        %timing: GTX580 (512 cores 772 MHz): finding corr. points - 37.63 s, batch total - 68.42 s
        %1 core 4GHz CPU (i7 990X): find corr. p. - 1412.66 s , batch - 1570.53 s
        %GPU acceleration - 37.54 times
t = tic;
Param.DestinationDir = GetDestinationDir(Param);   %removes "_original"
Param.SettingsDir = GetSettingsDir(Param.StartDir); %creates "Settings" at the same level as "Data_original"
if nargin==0
  Name = GetName(Param.StartDir); %Name = 'r17y20_08122013_t070000';   %for BackPack
end
%
Param.Name = Name;
Param.BestAnimal = 1;   %1 - NI; 2 - BP
if Param.Playback
  NI_BP.Channels = ...
   {'Wall microphone'; 'Playback'; 'Head microphone'; 'Accelerometer';...
    'NI sync'; 'BP sync'; 'HeadMicSongDet'; 'AccelSongDet'};
else
  NI_BP.Channels = ...
   {'Wall microphone'; 'Head microphone'; 'Accelerometer';...
    'NI sync'; 'BP sync'; 'HeadMicSongDet'; 'AccelSongDet'};  
end 
%BP.Range = 1;    %ADC range is +/-1V;
disp('Decoding backpack binary record...');
Ext = 'dat';
Name_tmp = [Param.StartDir filesep Name,'.',Ext];
tic
%[B, Sync, Vcount] = Datalogger4ChConverter_ErrInsensitiveSound(Name_tmp);
[B, Sync, Vcount] = Datalogger4ChConverter_ErrInsensitiveSound_mex(Name_tmp);
disp(['Error-free pieces: ' num2str(Vcount)]);
toc
%Datalogger4ChConverter_ErrInsensitiveSound;
%%
disp('Merging four channels to two...');
tic
Length_Sync = length(Sync);
disp(['Samples: ' num2str(Length_Sync)]);
C = Convert_4Ch_to_2Ch(B, Length_Sync);%this accelerate a bit from 
%C = Convert_4Ch_to_2Ch_mex(B, Length_Sync); %MEX is slower!
%Convert_4Ch_to_2Ch; %the result is in ADc counts, but in single() format; mean is 1024, range appr. 1024 +/- 512*(101/11) 
toc
%load([Param.StartDir filesep Name '_C']);  %here are C and Sync
%%
Cm = mean(C);
C = C-repmat(Cm,size(C,1),1); %here the mean of C was 1024 but range was from 512 to 1024+512
%BP.C = C/2^9*BP.Range;  %will be in Volts -  not a good idea, it should be
%int16
BP.C = C; %now "single", it is better to do int16() at the very end to decrease rounding errors
BP.Sync = Sync'; %uint8
clear C Sync Cm %temporarily removed
%as a result we get two arrays: C - 178804053x2 single; MIC, Accel
%                               Sync - 178804053x1 uint8
%
Param.MicChannel = 1;  %microphone channel
Param.PlaybackChannel = 3; %playback
Param.scanrateBP = 19200;  %BackPack sampling rate, sps
Param.scanrateMax = 32000;
%Param.scanrateDesired = 19200;%32000;
Param.InputRange = 5; %64/10.1; %in volts for backpack, but common for both devices (NI it is 5.0 V)
%Param.SongDetectionThreshold = [2 4/3 0.5]; %Recorder, Head microphone, Accelerometer
              %value for Recorder is given for refernce and is not used
Param.SongDetectionType = 0;    %0 - slassical for zebra finch harmonics+amplitude
                                %1 - only harmonics using autocorrelation,
                                %good for marmosets
if Param.SongDetectionType==0                               
  Param.SongDetectionThreshold = [2 4/3/2 0.5/4];              
else  
  Param.SongDetectionThreshold = [1 1 1]*0.2; %default was 0.6; 0.3 - more noise but may be OK, 0.1 - all
end                
Param.BuffersAbovePositiveAll = [5 20];    %in 4 ms buffers, default values, good for calls
Param.Threshold = 250;    %in points, one pulse - 800uS or 15.36 points, every 4 ms
                   %may be one pulse is OK to catch synchronization...
                   %10 - bad, 250 probably was better
Param.Pre = 2; %two seconds before detection
Param.Post = 1; %one second after detection
%if Param.scanrateBP~=Param.scanrateDesired - now always resample to Param.scanrateMax
if Param.LVDFile || (max(Param.scanrateDesired)>19200) || min(Param.SongSelection)<4 
                                %for WAV also needed
                                %for consistency between different options run separately
  disp('Resampling backpack record to 32kHz...');
  BP = ResampleBP(BP,Param);  %out C - single, Sync - uint8
end  
%end 
%
%detect songs by the head microphone and then by the accelerometer channel
%create plugin object
SongDet = fb_plugin_detect_song_harmonics2A;
%
BP.SongDetection = zeros(length(BP.Sync),2,'uint8');
if min(Param.SongSelection)<4   %i.e. song detection is needed 
disp('Detecting songs in HeadMic and Accel channels...');
tic
for J_t =1:2    %HeadMic, Accel
  %tic  
  if Param.SongDetectionType==0
  harmonics2 = SongDet.calc_harmonics2(BP.C(:,J_t)'/2^15*Param.InputRange,...
                                     SongDet.buffersize,SongDet.overlap,...
                                     SongDet.nonoverlap,SongDet.samplestart);  
  else
  [~, harmonics2] = harmonics_detect(BP.C(:,J_t)'/2^15*Param.InputRange,...
                                     SongDet.buffersize/2,SongDet.overlap-256,...
                                     '', 0, 40,...
                                     SongDet.nonoverlap,SongDet.samplestart);      
  end    
  %toc      % 27.738 sec                         
  %tic
songdetection = DetectSongs(harmonics2,Param,J_t);
  %toc       % 5.072 sec

% figure(1000); clf; plot(BP.harmonics2);
% hold on
% plot(single(songdetection)/10,'g');
%tic
PulseLength = Param.scanrateMax/1000; %1 ms  like recorder does, every 4 ms
s = repmat(songdetection',1,PulseLength);
s = [s zeros(size(s,1),128-PulseLength,'uint8')];
s = reshape(s',1,[]);
s((length(BP.Sync)+1):end) = []; %just for safety
BP.SongDetection(1:length(s),J_t) = s;
end
clear songdetection harmonics2 s
toc
end
%
if Param.Synchronization
  disp('Importing NI data...');
  [NI Param] = ConvertCurrentDirLVDtoArray(Param); %National Instruments, LVDs are taken from the Current directory
  %
  disp('Synchronizing backpack with NI...');  
  [NI_BP NI BP Param] = SynchronizeNI_BP(NI,BP,Param,NI_BP);
else
  if Param.Playback  
    NI_BP.Data = [int16(BP.C(:,1)) int16(BP.C(:,1)) int16(BP.C) int16(BP.Sync') int16(BP.Sync') int16(BP.SongDetection)]; 
  else
    NI_BP.Data = [int16(BP.C(:,1)) int16(BP.C) int16(BP.Sync') int16(BP.Sync') int16(BP.SongDetection)];   
  end    
  %we put false data at the first position where wall microphone should be
        %- just copy backpack microphone data
  %we put at NI synchronization data synchronization data from BP      
  %[int16(CA{1}) int16(CA{2}(:,1:(end-2)))...
  %   int16(SyncA{1}) int16(SyncA{2})' int16(CA{2}(:,end+[-1 0]))]; 
end
%clear NI BP %temporarily removed clearing
disp(num2str(size(NI_BP.Data)));
%disp('Saving intermediate results...');
%tic
%save([Param.StartDir filesep Param.Name '_NI_BP'],'-v7.3','NI_BP','Param');
%toc
%Here we have got 32kHz aligned array and now have to create different files from it...    
%
disp('Find new files borders...');
tic
[NI_BP Param] = GetNewFilesStartStop(NI_BP, Param);
disp('N new LVD/WAV files: ');
disp(num2str(NI_BP.Fcount));
toc    %70.927 seconds
%
disp('Creating new LVD/WAV files...');
tic
[NI_BP Param] = CreateLVDs(NI_BP, Param); 
toc   %967.204827 seconds
toc(t);
end
%Convertion of 1 GB file to WAV
%Decoding backpack binary record...                 Elapsed time is 264.656665 seconds.
%Merging four channels to two...                    Elapsed time is 184.786014 seconds.
%Resampling backpack record to 32kHz... (!)         Elapsed time is 95.655914 seconds.
%Detecting songs in HeadMic and Accel channels...(!)Elapsed time is 130.985472 seconds.
%Find new files borders...                          Elapsed time is 0.084731 seconds.
%Creating new LVD/WAV files...                      Elapsed time is 363.544534 seconds.

%Total Elapsed time is 1056.438301 seconds.

%When unnecessary things (!) are removed
%Elapsed time is 716.421476 seconds.
