classdef fb_plugin_detect_song_harmonics2A < fb_abstract.plot_buffered & fb_abstract.tag
    % plugin calculating a ratio on the harmonic power spectrum to separate
    % song from noise
    % 2010-10-28: changed to log-value. good separator is 3, range ~ [0,6]
    
    %% public methods
    methods
        
        %% constructor
        function obj = fb_plugin_detect_song_harmonics2A()
            obj.Name = 'Harmonics2 (SPL & harmonic power spectrum)';
            obj.Description = 'SPL based on harmonic power spectrum';
            obj.Author = 'Joshua Herbst';
            obj.plot_color = 'm';
            obj.LineWidth = 2;
            obj.buffersize = 4 * 128;
            obj.nonoverlap = 128;
            obj.samplestart = obj.buffersize;
        end
        
    end
    
    %% hidden methods
    methods (Access = protected, Hidden = true)
        
        %% transform function
        function plugin_transform_function(obj, Data)
            obj.transformed_data = obj.calc_harmonics2(Data, obj.buffersize, obj.overlap, obj.nonoverlap, obj.samplestart);
        end
    end
    
    %% STATIC methods
    methods (Static = true)
        
        function [harmonics2, buff_length] = calc_harmonics2(data, buffersize, overlap, nonoverlap, samplestart)
            % takes data, splits it into buffers and calcs the ratio for each buffer.
            
            %% buffer data, don't prepad with zeros.
            buff = buffer(data, buffersize, overlap, 'nodelay');
            
            %% calculate the number of buffers that we want to keep
            nbr_samples = size(data, 2);
            nbr_buffers_to_keep = fb_abstract.plot_buffered.calc_num_buffs(nbr_samples, samplestart, nonoverlap);
            buff = buff(:, 1:nbr_buffers_to_keep);

            %%
            buff_length = size(buff, 2);
            harmonics2 = zeros(1, buff_length);
            
            range_extension = 0;
            indices = zeros(4,128+range_extension);
            
            hps_factor = 4;
            for i = 1:hps_factor
                indices(i,:) = round((0:127+range_extension)/i)+1;
            end
            
%             tic    Array functions are worth than parfor!!!!
%             %H = repmat(hanning(512),1,buff_length);
%             HB = bsxfun(@times,hanning(512),buff);
%             F = fft(HB);  clear HB
%             F = F(1:128+range_extension,:);
%             buff_powerA = log(F.*conj(F)+0.1)-log(0.1)+1;  clear F
%             
%             b = buff_powerA(indices,:);  clear buff_powerA
%             b = reshape(b,4,128,[]);
%             harm_powerA = squeeze(prod(b));  clear b
%             [max_tmpA, max_indA] = max(harm_powerA(8*hps_factor+1:end,:));
%             min_tmpA = zeros(size(max_tmpA));            
%             parfor i = 1 : buff_length
%               min_tmpA(i) = min(harm_powerA((1:max_indA(i))+8*hps_factor,i));
%             end             
%             harmonics2 = log(max_tmpA./min_tmpA);
%             toc
            
            H = hanning(512);
            
            parfor i = 1 : buff_length                
                  %replaced by Alexei 17.02.2014  
%                 buff_power = log(abs(fft2(hanning(512).* buff(:,i)))+0.1);
%                 buff_power = buff_power(1:128+range_extension);
%                 buff_power = log(((exp(buff_power)-0.1).^2)+0.1)-log(0.1)+1;                
                F = fft(H.* buff(:,i));
                Ft = F(1:128+range_extension);                
                buff_power = log(Ft.*conj(Ft)+0.1)-log(0.1)+1;  
                
%                 tmp = 0;   %removed by Alexei 17.02.2014
%                 buff_power(1:tmp) = buff_power(1:tmp)+(tmp:-1:1)'; 
                
                harm_power = prod(buff_power(indices),1);
                [max_tmp, max_ind] = max(harm_power(8*hps_factor+1:end));
                min_tmp = min(harm_power((1:max_ind)+8*hps_factor));
                harmonics2(i) = log(max_tmp/min_tmp);                
            end
            
        end
        
    end
    
end
%% EOF
