function [harmonics, harmonics_raw] = harmonics_detect(song, buffersize, overlap, threshold, calc_binary_output, samples_for_acf, nonoverlap, samplestart)
% buffer the song into segments of 256 samples and computes the
% autocorrelation on these buffers. harmonics (typical for zebra finch
% song) result in nice oscillating values for the acf. the arbitrary
% threshold seems to work fine...

    %% use reasonable default values if none are given.
    
    if nargin < 8
        samplestart = buffersize;
    end
    
    if nargin < 7
        nonoverlap = (buffersize-overlap);
    end    
    
    
    if nargin == 5
        samples_for_acf = 40;
    end

    if nargin == 4
        calc_binary_output = 1;
        samples_for_acf = 40;
    end

    if nargin == 3
        threshold = 0.6;
        % do the thresholding by default (needed in analyze)
        calc_binary_output = 1;
        samples_for_acf = 40;
    end
    
   %% buffer data, don't prepad with zeros.
    buffered_song = buffer(song, buffersize, overlap, 'nodelay');
    
    %% calculate the number of buffers that we want to keep
    nbr_samples = size(song, 2);
    nbr_buffers_to_keep = fb_abstract.plot_buffered.calc_num_buffs(nbr_samples, samplestart, nonoverlap);
    buffered_song = buffered_song(:, 1:nbr_buffers_to_keep);

    %%
    
    number_of_buffers = size(buffered_song, 2);

    % initialize vector differences between envelope of acf and acf...
    harmonics_raw = zeros(1, number_of_buffers);
    harmonics = harmonics_raw;

    %% fill the autocorrelation matrix and calculate envelopes
    
    for count = 1 : number_of_buffers
        harmonics_raw(count) = ...
            containing_area(autocorr(buffered_song(:, count), samples_for_acf-1)');
    end

    %% convert harmonics_raw into a scale between 1 and 0.
    harmonics_raw = harmonics_raw ./ samples_for_acf;
    
    %% determine whether song or not - if requested
    
    if calc_binary_output == 1
        
        % the threshold (0-1) depends on how many samples were used for the ACF
        harmonics(harmonics_raw > threshold) = 1;
        
        %     soundsc(song,32000)
        %     figure(81351);clf;plot(harmonics_raw/samples_for_acf);hold on;plot([1 number_of_buffers],[0.6 0.6],'k');ylim([0 1])
        %     detect_song(song)
    end
    
    
end

%% HELPERFUNCTIONS

function area = containing_area(data)
% calculate the amount of water that a you could pour into a
% curve... determine the smallest non-rising curve that is larger
% than the data and take the difference.

    % dimensions of data
    numel = size(data, 2);

    % determine smallestnon-rising envelope
    envelope = data;
    
    % start at the very and move buffer by buffer till the beginning
    for count = numel-1 : -1 : 1
        envelope(count) = max(data(count), envelope(count+1));
    end

    % take difference between envelope and data and sum
%     disp(envelope - data)
    area = sum(envelope - data);
    
end

%% EOF